package th.co.gosoft.voip_sendbird_plugin;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.sendbird.calls.AcceptParams;
import com.sendbird.calls.CallOptions;
import com.sendbird.calls.DialParams;
import com.sendbird.calls.DirectCall;
import com.sendbird.calls.SendBirdCall;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

import app.loup.streams_channel.StreamsChannel;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;

import th.co.gosoft.voip_sendbird_plugin.service.CallService;
import th.co.gosoft.voip_sendbird_plugin.handler.AuthHandler;
import th.co.gosoft.voip_sendbird_plugin.handler.RingingHandler;
import th.co.gosoft.voip_sendbird_plugin.handler.DirectCallHandler;

/**
 * VoipSendbirdPlugin
 */
public class VoipSendbirdPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    private MethodChannel channel;
    private Context context;
    private Activity activity;
    public static final String TAG = "Sendbird VOIP";

    private CallService mCallService = new CallService();

    private static AuthHandler authHandler = new AuthHandler();
    private static RingingHandler ringingHandler = new RingingHandler();
    private static DirectCallHandler directCallHandler = new DirectCallHandler();

    private static final String successStatus = "Success";

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        this.context = flutterPluginBinding.getApplicationContext();
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "voipSendbirdPlugin");
        channel.setMethodCallHandler(this);

        final StreamsChannel authChannel = new StreamsChannel(flutterPluginBinding.getBinaryMessenger(), "AuthChannel");
        authChannel.setStreamHandlerFactory(new StreamsChannel.StreamHandlerFactory() {
            @Override
            public EventChannel.StreamHandler create(Object arguments) {
                return authHandler;
            }
        });

        final StreamsChannel ringingChannel = new StreamsChannel(flutterPluginBinding.getBinaryMessenger(), "RingingChannel");
        ringingChannel.setStreamHandlerFactory(new StreamsChannel.StreamHandlerFactory() {
            @Override
            public EventChannel.StreamHandler create(Object arguments) {
                return ringingHandler;
            }
        });

        final StreamsChannel directCallChannel = new StreamsChannel(flutterPluginBinding.getBinaryMessenger(), "DirectCallChannel");
        directCallChannel.setStreamHandlerFactory(new StreamsChannel.StreamHandlerFactory() {
            @Override
            public EventChannel.StreamHandler create(Object arguments) {
                return directCallHandler;
            }
        });
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("getPlatformVersion")) {
            result.success("Android " + android.os.Build.VERSION.RELEASE);
        } else if (call.method.equals("initApp")) {
            String applicationId = call.argument("applicationId").toString();
            onInitial(applicationId);
            result.success(successStatus);
        } else if (call.method.equals("signIn")) {
            String userId = call.argument("userId").toString();
            String fcmToken = call.argument("fcmToken").toString();
            onSignIn(userId, fcmToken);
            result.success(successStatus);
        } else if (call.method.equals("signOut")) {
            onSignOut();
            result.success(successStatus);
        } else if (call.method.equals("call")) {
            String calleeId = call.argument("calleeId").toString();
            makeCall(calleeId);
            result.success(successStatus);
        } else if (call.method.equals("toggleAudioDevice")) {
            onChangeAudioDevice();
            result.success(successStatus);
        } else if (call.method.equals("endCall")) {
            endCall();
            result.success(successStatus);
        } else if (call.method.equals("acceptCall")) {
            acceptCall();
            result.success(successStatus);
        } else if (call.method.equals("onMessageReceived")) {
            String message = call.argument("remoteMessage").toString();
            Map<String, String> data = new Gson().
                    fromJson(message, new TypeToken<HashMap<String, Object>>() {
                    }.getType());
            onMessageReceived(data);
            System.out.println(data);
            result.success(successStatus);
        } else {
            result.notImplemented();
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    public void onMessageReceived(Map<String, String> message) {
        if (SendBirdCall.handleFirebaseMessageData(message)) {
            Log.i(TAG, "[FirebaseMessagingService] onMessageReceived() => " + message.toString());
        }
    }

    public void onInitial(String appId) {
        ringingHandler.onInitial(context, appId, mCallService);
    }

    public void onSignIn(String userId, String fcmToken) {
        authHandler.onSignIn(userId, fcmToken);
    }

    public void onSignOut() {
        authHandler.onSignOut();
    }

    public void onChangeAudioDevice() {
        DirectCall mDirectCall = mCallService.getDirectCall();
        directCallHandler.setAudioDevice(mDirectCall);
    }

    public void makeCall(String CalleeId) {
        DialParams dialParams = new DialParams(CalleeId);
        dialParams.setVideoCall(false);

        CallOptions callOptions = new CallOptions();
        callOptions.setAudioEnabled(true);
        callOptions.setVideoEnabled(false);

        dialParams.setCallOptions(callOptions);

        SendBirdCall.dial(dialParams, (directCall, e) -> {
            if (e != null) {
                Log.i(TAG, "[MakeCall] dial -> " + e.getMessage());
            }
            Log.i(TAG, "[MakeCall] dial -> success");
            mCallService.setDirectCall(directCall);
            directCallHandler.setCallListener(mCallService.getDirectCall());
        });
    }

    public void endCall() {
        DirectCall mDirectCall = mCallService.getDirectCall();
        mDirectCall.end();
    }

    public void acceptCall() {
        CallOptions callOptions = new CallOptions();
        callOptions.setAudioEnabled(true);
        callOptions.setVideoEnabled(false);

        DirectCall mDirectCall = mCallService.getDirectCall();
        mDirectCall.accept(new AcceptParams().setCallOptions(callOptions));
    }


    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        this.activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }
}
