package th.co.gosoft.voip_sendbird_plugin.handler;

import android.content.Context;

import com.sendbird.calls.AuthenticateParams;
import com.sendbird.calls.SendBirdCall;

import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;

public class AuthHandler implements EventChannel.StreamHandler {
    private String TAG = "Sendbird VOIP [AuthHandle]";

    public static final String STATE_CONNECTING = "1000";
    public static final String STATE_LOGIN = "0000";
    public static final String STATE_LOGOUT = "4001";
    public static final String STATE_CONNECT_FAILED = "5005";

    private EventChannel.EventSink eventSink;

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.eventSink = events;
    }

    @Override
    public void onCancel(Object arguments) {
    }

    public void onSignIn(String userId, String fcmToken) {
        eventSink.success(STATE_CONNECTING);
        AuthenticateParams authenticateParams = new AuthenticateParams(userId);
        SendBirdCall.deauthenticate(e -> {
            if (e != null) {
                Log.i(TAG, "[OnSignIn] deauthenticate -> " + e.getMessage());
            }

            SendBirdCall.authenticate(authenticateParams, ((user, e1) -> {
                if (e1 != null) {
                    Log.i(TAG, "[OnSignIn] authenticate -> " + e1.getMessage());
                    eventSink.error("" + e1.getCode(), e1.getMessage(), STATE_CONNECT_FAILED);

                } else {
                    Log.i(TAG, "[OnSignIn] authenticate -> success");
                    SendBirdCall.registerPushToken(fcmToken, false, e2 -> {
                        if (e2 != null) {
                            Log.i(TAG, "[OnSignIn] registerPushToken -> " + e2.getMessage());
                            eventSink.error("" + e2.getCode(), e2.getMessage(), STATE_CONNECT_FAILED);
                        } else {
                            Log.i(TAG, "[OnSignIn] registerPushToken -> success");
                            eventSink.success(STATE_LOGIN);
                        }
                    });
                }
            }));
        });
    }

    public void onSignOut() {
        eventSink.success(STATE_CONNECTING);
        SendBirdCall.deauthenticate(e -> {
            if (e != null) {
                Log.i(TAG, "[OnSignIn] deauthenticate -> " + e.getMessage());
                eventSink.error("" + e.getCode(), e.getMessage(), STATE_CONNECT_FAILED);
            } else {
                eventSink.success(STATE_LOGOUT);
            }

        });
    }
}