package th.co.gosoft.voip_sendbird_plugin.handler;

import com.sendbird.calls.AudioDevice;
import com.sendbird.calls.DirectCall;
import com.sendbird.calls.handler.DirectCallListener;

import java.util.Set;

import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;

public class DirectCallHandler implements EventChannel.StreamHandler {
    private String TAG = "Sendbird VOIP [DirectCallHandle]";

    public static final String STATE_CONNECTING = "1000";
    public static final String STATE_RINGING = "2000";
    public static final String STATE_CONNECTED = "0000";
    public static final String STATE_ENDED = "4000";
    public static final String STATE_CONNECT_FAILED = "5005";

    public static final String STATE_CHANGING_DEVICE = "3001";
    public static final String STATE_CHANGE_DEVICE_SUCCESS = "3000";
    public static final String STATE_CHANGE_DEVICE_FAILED = "3002";

    private EventChannel.EventSink eventSink;

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.eventSink = events;
    }

    @Override
    public void onCancel(Object arguments) {
    }

    public void setCallListener(DirectCall mDirectCall) {
        eventSink.success(STATE_CONNECTING);
        mDirectCall.setListener(new DirectCallListener() {

            @Override
            public void onConnected(DirectCall call) {
                Log.i(TAG, "[OnDialListener] onConnected");
                eventSink.success(STATE_CONNECTED);
            }

            @Override
            public void onEnded(DirectCall call) {
                Log.i(TAG, "[OnDialListener] onEnded");
                eventSink.success(STATE_ENDED);
            }

            @Override
            public void onAudioDeviceChanged(DirectCall call, AudioDevice currentAudioDevice, Set<AudioDevice> availableAudioDevices) {
                Log.i(TAG, "[OnDialListener] onAudioDeviceChanged");
                call.unmuteMicrophone();
                eventSink.success(STATE_RINGING);
            }
        });
    }

    public void setAudioDevice(DirectCall call) {
        eventSink.success(STATE_CHANGING_DEVICE);
        AudioDevice audioDevice = call.getCurrentAudioDevice();

        if (audioDevice == AudioDevice.SPEAKERPHONE) {
            Log.i(TAG, "[setAudioDevice] AudioDevice.EARPIECE");
            call.selectAudioDevice(AudioDevice.EARPIECE, e -> {
                if (e != null) {
                    Log.i(TAG, "[setAudioDevice] Failed -> " + e.getMessage());
                    eventSink.error("" + e.getCode(), e.getMessage(), STATE_CHANGE_DEVICE_FAILED);
                }
                eventSink.success(STATE_CHANGE_DEVICE_SUCCESS);
            });
        } else {
            Log.i(TAG, "[setAudioDevice] AudioDevice.SPEAKERPHONE");
            call.selectAudioDevice(AudioDevice.SPEAKERPHONE, e -> {
                if (e != null) {
                    Log.i(TAG, "[setAudioDevice] Failed -> " + e.getMessage());
                    eventSink.error("" + e.getCode(), e.getMessage(), STATE_CHANGE_DEVICE_FAILED);
                }
                eventSink.success(STATE_CHANGE_DEVICE_SUCCESS);
            });
        }
    }
}