package th.co.gosoft.voip_sendbird_plugin.handler;

import android.content.Context;

import com.sendbird.calls.DirectCall;
import com.sendbird.calls.SendBirdCall;
import com.sendbird.calls.handler.DirectCallListener;
import com.sendbird.calls.handler.SendBirdCallListener;

import java.util.UUID;

import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;
import th.co.gosoft.voip_sendbird_plugin.service.CallService;

public class RingingHandler implements EventChannel.StreamHandler {
    private String TAG = "Sendbird VOIP [RingingHandle]";

    public static final String STATE_INITIALING = "1001";
    public static final String STATE_INITIALED = "1000";
    public static final String STATE_RINGING = "2002";
    public static final String STATE_CONNECTED = "0000";
    public static final String STATE_ENDED = "4000";
    public static final String STATE_CONNECT_FAILED = "5005";

    private EventChannel.EventSink eventSink;

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.eventSink = events;
    }

    @Override
    public void onCancel(Object arguments) {
    }

    public void onInitial(Context context, String applicationId, CallService mCallSerice) {
        eventSink.success(STATE_INITIALING);
        if (SendBirdCall.init(context, applicationId)) {
            SendBirdCall.removeAllListeners();
            SendBirdCall.addListener(UUID.randomUUID().toString(), new SendBirdCallListener() {
                @Override
                public void onRinging(DirectCall call) {
                    int ongoingCallCount = SendBirdCall.getOngoingCallCount();
                    Log.i(TAG, "onRinging() => callId: " + call.getCallId() + ", getOngoingCallCount(): " + ongoingCallCount);
                    if (ongoingCallCount > 1) {
                        call.end();
                    } else {
                        mCallSerice.setDirectCall(call);
                        call.setListener(new DirectCallListener() {
                            @Override
                            public void onConnected(DirectCall call) {
                                Log.i(TAG, "onConnected() => callId: " + call.getCallId());
                                eventSink.success(STATE_CONNECTED);
                            }

                            @Override
                            public void onEnded(DirectCall call) {
                                int ongoingCallCount = SendBirdCall.getOngoingCallCount();
                                Log.i(TAG, "onEnded() => callId: " + call.getCallId() + ", getOngoingCallCount(): " + ongoingCallCount);
                                eventSink.success(STATE_ENDED);
                            }
                        });

                        Log.i(TAG, "on Ringing....");
                        eventSink.success(STATE_RINGING);
                    }
                }
            });

//            SendBirdCall.Options.addDirectCallSound(SendBirdCall.SoundType.DIALING, R.raw.dialing);
//            SendBirdCall.Options.addDirectCallSound(SendBirdCall.SoundType.RINGING, R.raw.ringing);
//            SendBirdCall.Options.addDirectCallSound(SendBirdCall.SoundType.RECONNECTING, R.raw.reconnecting);
//            SendBirdCall.Options.addDirectCallSound(SendBirdCall.SoundType.RECONNECTED, R.raw.reconnected);
            eventSink.success(STATE_INITIALED);
        }
    }

}