import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:streams_channel/streams_channel.dart';

class VoipSendbirdPlugin {
  static const MethodChannel _channel =
      const MethodChannel('voipSendbirdPlugin');
  static StreamsChannel _authChannel = StreamsChannel('AuthChannel');
  static StreamsChannel _ringingChannel = StreamsChannel('RingingChannel');
  static StreamsChannel _directCallChannel =
      StreamsChannel('DirectCallChannel');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static Future<String> initApp(String applicationId) async {
    Map<String, dynamic> args = <String, dynamic>{};
    args.putIfAbsent("applicationId", () => applicationId);
    final String connectResult = await _channel.invokeMethod('initApp', args);
    return connectResult;
  }

  static Future<String> signIn(String userId, String fcmToken) async {
    Map<String, dynamic> args = <String, dynamic>{};
    args.putIfAbsent("userId", () => userId);
    args.putIfAbsent("fcmToken", () => fcmToken);
    final String connectResult = await _channel.invokeMethod('signIn', args);
    return connectResult;
  }

  static Future<String> signOut() async {
    Map<String, dynamic> args = <String, dynamic>{};
    final String connectResult = await _channel.invokeMethod('signOut', args);
    return connectResult;
  }

  static Future<String> call(String calleeId) async {
    Map<String, dynamic> args = <String, dynamic>{};
    args.putIfAbsent("calleeId", () => calleeId);
    final String connectResult = await _channel.invokeMethod('call', args);
    return connectResult;
  }

  static Future<String> toggleAudioDevice() async {
    Map<String, dynamic> args = <String, dynamic>{};
    final String connectResult =
        await _channel.invokeMethod('toggleAudioDevice', args);
    return connectResult;
  }

  static Future<String> endCall() async {
    Map<String, dynamic> args = <String, dynamic>{};
    final String connectResult = await _channel.invokeMethod('endCall', args);
    return connectResult;
  }

  static Future<String> acceptCall() async {
    Map<String, dynamic> args = <String, dynamic>{};
    final String connectResult =
        await _channel.invokeMethod('acceptCall', args);
    return connectResult;
  }

  static Future<String> onMessageReceived(String remoteMessage) async {
    Map<String, dynamic> args = <String, dynamic>{};
    args.putIfAbsent("remoteMessage", () => remoteMessage);
    final String connectResult =
        await _channel.invokeMethod('onMessageReceived', args);
    return connectResult;
  }

  // ========================== Stream Handler =================================
//  static void getAuthListener(String argument) {
//    print('begin:authListener');
//    WidgetsFlutterBinding.ensureInitialized();
//    _authChannel.receiveBroadcastStream(argument).listen((i) {
//
//    });
//  }

  static Stream<String> authListener(String argument) {
    WidgetsFlutterBinding.ensureInitialized();
    return _authChannel
        .receiveBroadcastStream(argument)
        .map<String>((v) => (v));
  }

  static Stream<String> ringingListener(String argument) {
    WidgetsFlutterBinding.ensureInitialized();
    return _ringingChannel
        .receiveBroadcastStream(argument)
        .map<String>((v) => (v));
  }

  static Stream<String> directCallListener(String argument) {
    WidgetsFlutterBinding.ensureInitialized();
    return _directCallChannel
        .receiveBroadcastStream(argument)
        .map<String>((v) => (v));
  }
}
